@@ -1,6 +1,6 @@
 from termcolor import colored
 from functools import reduce
-from typing import List, Optional
+from typing import List
 from model.common.address import Address
 from model.common.phone import Phone
 from model.common.trperson import (
@@ -10,20 +10,21 @@ from model.common.trperson import (
     Marriage,
     Education,
     Employment,
-    Family,
 )
+import json
+
 from model.common.tr import TrCaseIn, TrBackground, SpInCanada
-from model.common.commonmodel import CommonModel
+from model.common.commonmodel import CommonModel, BuilderModel
+from pdfform.tr.fb5709 import FormBuilder5709
 
 
-class M5709Model(CommonModel):
+class M5709Model(CommonModel, BuilderModel):
     personal: Personal
     marriage: Marriage
     personid: List[PersonId]
     address: List[Address]
     education: List[Education]
     employment: List[Employment]
-    family: List[Family]
     phone: List[Phone]
     cor: List[COR]
     trcasein: TrCaseIn
@@ -40,3 +41,16 @@ class M5709Model(CommonModel):
                     "You must input excel file list as source data for validation"
                 )
         super().__init__(excels, output_excel_file, globals())
+
+    def make_pdf_form(self, output_json, *args, **kwargs):
+        pf = FormBuilder5709(self)
+        form = pf.get_form()
+        with open(output_json, "w") as output:
+            json.dump(form.actions, output, indent=3, default=str)
+        print(colored(f"{output_json} has been created. ", "green"))
+
+    def make_web_form(self, output_json, upload_dir, rcic, *args, **kwargs):
+        raise ValueError("This model doesn't have webform...")
+
+    def context(self, *args, **kwargs):
+        raise ValueError("This model doesn't have webform...")
