@@ -8,20 +8,23 @@ from model.common.trperson import (
     Marriage,
     Education,
     Employment,
-    Family,
+    # Family,
 )
 from model.common.tr import TrCaseIn, TrBackground, WpInCanada
-from model.common.commonmodel import CommonModel
+from model.common.commonmodel import CommonModel, BuilderModel
+from pdfform.tr.fb5710 import FormBuilder5710
+from termcolor import colored
+import json
 
 
-class M5710Model(CommonModel):
+class M5710Model(CommonModel, BuilderModel):
     personal: Personal
     marriage: Marriage
     personid: List[PersonId]
     address: List[Address]
     education: List[Education]
     employment: List[Employment]
-    family: List[Family]
+    # family: List[Family]
     phone: List[Phone]
     cor: List[COR]
     trcasein: TrCaseIn
@@ -38,3 +41,16 @@ class M5710Model(CommonModel):
                     "You must input excel file list as source data for validation"
                 )
         super().__init__(excels, output_excel_file, globals())
+
+    def make_pdf_form(self, output_json, *args, **kwargs):
+        pf = FormBuilder5710(self)
+        form = pf.get_form()
+        with open(output_json, "w") as output:
+            json.dump(form.actions, output, indent=3, default=str)
+        print(colored(f"{output_json} has been created. ", "green"))
+
+    def make_web_form(self, output_json, upload_dir, rcic, *args, **kwargs):
+        raise ValueError("This model doesn't have webform...")
+
+    def context(self, *args, **kwargs):
+        raise ValueError("This model doesn't have webform...")
