@@ -80,21 +80,14 @@ job_duration_unit = {"validate": "list", "source": ["day", "week", "month", "yea
 purpose_of_lmia = {
     "validate": "list",
     "source": [
-        "Support Permanent Resident only",
-        "Support Work Permit only",
-        "Support both Work Permit and Permanent Resident",
+        "Supporting Permanent Resident only",
+        "Supporting Work Permit only",
+        "Supporting both Work Permit and Permanent Resident",
     ],
 }
 stream_of_lmia = {
     "validate": "list",
-    "source": [
-        "1. EE(Express Entry: 5593)",
-        "2. HWS(High Wage Stream: 5626)",
-        "3. LWS(Low Wage Stream: 5627)",
-        "4. GTS(Global Talent Stream: 5624, 5625)",
-        "5. AC (Academic: 5626)",
-        "6. AG (Agriculture: 5519, 5510)",
-    ],
+    "source": ["EE", "HWS", "LWS", "GTS", "AC", "AG", "CG"],
 }
 rent_unit = {"validate": "list", "source": ["week", "month"]}
 accommodation_type = {
@@ -114,6 +107,7 @@ workpermit_type = {
         "Other",
         "Post Graduation Work Permit",
         "Start-up Business Class",
+        # "International Experience Canada (IEC)",
     ],
 }
 marital_status = {
@@ -144,6 +138,14 @@ education_level = {
         "Less than high school",
     ],
 }
+trade_education_type = {
+    "validate": "list",
+    "source": [
+        "Apprenticeship diploma/certificate",
+        "Trade diploma/certificate",
+        "Vocational school diploma/certificate",
+    ],
+}
 relationship = {
     "validate": "list",
     "source": [
@@ -234,6 +236,20 @@ tr_original_purpose = {
     "source": ["Business", "Tourism", "Study", "Work", "Other", "Family Visit"],
 }
 
+visit_purpose_5257 = {
+    "validate": "list",
+    "source": [
+        "Business",
+        "Tourism",
+        "Short-Term Studies",
+        "Returning Student",
+        "Returning Worker",
+        "Super Visa: For Parents or Grandparents",
+        "Other",
+        "Family Visit",
+    ],
+}
+
 date_format = {
     "validate": "date",
     "criteria": ">",
@@ -248,7 +264,13 @@ positive_int = {
     "input_title": "请输入整数:",
     "input_message": "必须是大于0",
 }
-
+positive_int_include_zero = {
+    "validate": "integer",
+    "criteria": ">=",
+    "value": 0,
+    "input_title": "请输入整数:",
+    "input_message": "必须是大于等于0",
+}
 positive_int_decimal = {
     "validate": "decimal",
     "criteria": ">",
@@ -259,8 +281,9 @@ positive_int_decimal = {
 
 int_decimal = {
     "validate": "decimal",
-    "criteria": "",
-    # "value": 0,
+    "criteria": "between",
+    "minimum": -100000000000,
+    "maximum": 100000000000,
     "input_title": "请输入整数或小数:",
     "input_message": "无论正数负数",
 }
@@ -275,6 +298,91 @@ employment_type = {
     "source": ["Employed", "Self-employed"],
 }
 
+relationship_to_pa = {
+    "validate": "list",
+    "source": [
+        "Adopted Child",
+        "Child",
+        "Common-law partner",
+        "Grandchild",
+        # "Other": "5: 85",
+        "Spouse",
+        "Step-Child",
+        "Step-Grandchild",
+        "Parent",
+        "Adoptive Parent",
+    ],
+}
+
+dependant_type = {
+    "validate": "list",
+    "source": [
+        "Type A Dependant",
+        "Type B Dependant",
+        "Type C Dependant",
+    ],
+}
+
+eca_supplier = {
+    "validate": "list",
+    "source": [
+        "Comparative Education Service - University of Toronto School of Continuing Studies",
+        "International Credential Assessment Service of Canada",
+        "World Education Services",
+        "International Qualifications Assessment Service",
+        "International Credential Evaluation Service",
+        # "Medical Council of Canada (for Doctors)",
+        # "Pharmacy Examining Board of Canada (for Pharmacists)",
+    ],
+}
+
+language_remark = {
+    "validate": "list",
+    "source": ["Test Score", "Estimation"],
+}
+
+cap_exemption_type = {
+    "validate": "list",
+    "source": [
+        "Caregiver positions in a health care facility (NOC 3012, 3233, and 3413)",
+        "On-farm primary agricultural positions",
+        "Position for the Global Talent Stream",
+        "Position(s) is/are highly mobile",
+        "Position(s) is/are truly temporary",
+        "Seasonal 270-day exemption",
+    ],
+}
+
+
+months = {
+    "validate": "list",
+    "source": [
+        "January",
+        "February",
+        "March",
+        "April",
+        "May",
+        "June",
+        "July",
+        "August",
+        "September",
+        "October",
+        "November",
+        "December",
+    ],
+}
+
+waive_creteria = {
+    "validate": "list",
+    "source": [
+        "Caregiver positions in health care institutions",
+        "Limited duration positions",
+        "On-farm primary agricultural positions",
+        "Positions within a specialized occupation",
+        "Unique skills or traits",
+    ],
+}
+
 
 # Validation data
 validation = {
@@ -293,20 +401,20 @@ validation = {
     },
     # Employer
     "info-general": {
-        "ft_employee_number": positive_int,
-        "pt_pt_employee_number": positive_int,
+        "ft_employee_number": positive_int_include_zero,
+        "pt_employee_number": positive_int_include_zero,
         "establish_date": date_format,
         "has_lmia_approved": yes_no,
         "corporate_structure": corporate_structure,
         "num_pnps": positive_int,
         "num_pnps_approved": positive_int,
-        "num_pnps_in_process": positive_int,
+        "num_pnps_in_process": positive_int_include_zero,
         "when_lmia_approved": date_format,
         "last_revenue": positive_int_decimal,
-        "last_profit": positive_int_decimal,
-        "retained_earning": positive_int_decimal,
-        "before_last_revenue": positive_int_decimal,
-        "before_last_profit": positive_int_decimal,
+        "last_profit": int_decimal,
+        "retained_earning": int_decimal,
+        "before_last_revenue": int_decimal,
+        "before_last_profit": int_decimal,
         "has_jobbank_account": yes_no,
         "has_bc_employer_certificate": yes_no,
     },
@@ -325,16 +433,24 @@ validation = {
         "immigration_status": imm_status,
     },
     "info-position": {
+        "is_new": yes_no,
         "worked_working": yes_no,
         "under_cba": yes_no,
         "has_same": yes_no,
         "lowest": positive_int_decimal,
         "highest": positive_int_decimal,
         "lmia_refused": yes_no,
+        "has_same_number": positive_int,
+        "vacancies_number": positive_int_include_zero,
+        "laidoff_with12": positive_int_include_zero,
+        "laidoff_current": positive_int_include_zero,
     },
     "info-joboffer": {
         "license_request": yes_no,
+        "license_met": yes_no,
         "union": yes_no,
+        "atypical_schedule": yes_no,
+        "days": positive_int_decimal,
         "hours": positive_int_decimal,
         "payment_way": salary_payment_way,
         "wage_unit": wage_unit,
@@ -355,14 +471,16 @@ validation = {
         "extended_medical_insurance": yes_no,
         "english_french": yes_no,
         "other_language_required": yes_no,
-        "oral": positive_int_decimal,
-        "writing": positive_int_decimal,
+        "oral": english_or_french,
+        "writing": english_or_french,
         "offer_date": date_format,
         "vacation_pay_weeks": positive_int_decimal,
         "vacation_pay_percentage": positive_int_decimal,
+        "education_level": education_level,
+        "is_trade": yes_no,
+        "trade_type": trade_education_type,
     },
     # LMIA
-    "table-employee_list": {"immigration_status": imm_status},
     "info-lmiacase": {
         "area_index": positive_int,
         "province_index": positive_int,
@@ -377,26 +495,49 @@ validation = {
         "stream_of_lmia": stream_of_lmia,
         "has_another_employer": yes_no,
         "number_of_tfw": positive_int,
-        "duration_number": positive_int_decimal,
+        "duration_number": positive_int,
         "duration_unit": lmia_duration_unit,
-        "is_positive_within_2": yes_no,
-        "is_recent_positive": yes_no,
         "has_attestation": yes_no,
+        "use_jobbank": yes_no,
     },
     "info-lmi": {
         "laid_off_in_12": yes_no,
         "is_work_sharing": yes_no,
         "labour_dispute": yes_no,
+        "canadian_lost_job": yes_no,
+        "laid_off_canadians": positive_int,
+        "laid_off_tfw": positive_int,
     },
     "info-emp5624": {
         "hird_canadian": yes_no,
         "why_not": yes_no,
         "has_active_lmbp": yes_no,
     },
+    "table-emp5624lmbp": {"start_date": date_format, "end_date": date_format},
+    "info-emp5626": {
+        "is_seasonal": yes_no,
+        "start_month": months,
+        "end_month": months,
+        "last_canadian_number": positive_int,
+        "last_tfw_number": positive_int,
+        "current_canadian_number": positive_int,
+        "current_tfw_number": positive_int,
+        "tp_waivable": yes_no,
+        "has_finished_tp": yes_no,
+        "waive_creteria": waive_creteria,
+    },
     "info-emp5627": {
+        "named": yes_no,
+        "is_in_seasonal_industry": yes_no,
         "provide_accommodation": yes_no,
         "rent_unit": rent_unit,
         "accommodation_type": accommodation_type,
+        "rent_amount": positive_int_include_zero,
+        "bedrooms": positive_int,
+        "people": positive_int,
+        "bathrooms": positive_int,
+        "cap_exempted": yes_no,
+        "which_exemption": cap_exemption_type,
     },
     # PA
     "info-personal": {
@@ -415,16 +556,20 @@ validation = {
         "secondary_school_years": positive_int,
         "post_secondary_school_years": positive_int,
         "other_school_years": positive_int,
+        "relationship_to_pa": relationship_to_pa,
+        "accompany_to_canada": yes_no,
+        "dependant_type": dependant_type,
+        "eca_supplier": eca_supplier,
+        "ita_assessed": yes_no,
     },
     "table-personid": {"issue_date": date_format, "expiry_date": date_format},
     "info-status": {
         "current_country_status": imm_status,
-        "current_status_type": workpermit_type,
+        "current_workpermit_type": workpermit_type,
         "has_vr": yes_no,
         "current_status_start_date": date_format,
         "current_status_end_date": date_format,
         "last_entry_date": date_format,
-        "last_entry_place": date_format,
     },
     "info-ee": {"ee_expiry_date": date_format, "ee_score": positive_int},
     "info-marriage": {
@@ -454,8 +599,7 @@ validation = {
     "table-history": {"start_date": date_format, "end_date": date_format},
     "table-addresshistory": {"start_date": date_format, "end_date": date_format},
     "table-language": {
-        "test_type": language_test_type,
-        "date": date_format,
+        "test_date": date_format,
         "report_date": date_format,
         "reading": positive_int_decimal,
         "writting": positive_int_decimal,
@@ -569,9 +713,8 @@ validation = {
     },
     "info-trcase": {
         "service_in": english_french,
-        "application_purpose": tr_application_purpose,
         "same_as_cor": yes_no,
-        "applying_stauts": imm_status,
+        "applying_status": imm_status,
         "applying_start_date": date_format,
         "applying_end_date": date_format,
         "submission_date": date_format,
@@ -580,8 +723,8 @@ validation = {
         "study_level": education_level,
         "province": canada_provinces,
         "paid_person": sp_paid_person,
-        "duration_from": date_format,
-        "duration_to": date_format,
+        "start_date": date_format,
+        "end_date": date_format,
         "tuition_cost": positive_int_decimal,
         "room_cost": positive_int_decimal,
         "other_cost": positive_int_decimal,
@@ -589,9 +732,11 @@ validation = {
     },
     "info-vrincanada": {
         "application_purpose": vr_application_purpose,
-        "duration_from": date_format,
-        "duration_to": date_format,
+        "visit_purpose": tr_original_purpose,
+        "start_date": date_format,
+        "end_date": date_format,
         "funds_available": positive_int_decimal,
+        "paid_person": sp_paid_person,
     },
     "info-spincanada": {
         "application_purpose": sp_in_application_purpose,
@@ -600,8 +745,8 @@ validation = {
         "paid_person": sp_paid_person,
         "apply_work_permit": yes_no,
         "work_permit_type": sp_apply_wp_type,
-        "duration_from": date_format,
-        "duration_to": date_format,
+        "start_date": date_format,
+        "end_date": date_format,
         "tuition_cost": positive_int_decimal,
         "room_cost": positive_int_decimal,
         "other_cost": positive_int_decimal,
@@ -612,21 +757,22 @@ validation = {
         "application_purpose": wp_in_application_purpose,
         "work_province": canada_provinces,
         "work_permit_type": wp_apply_wp_type,
-        "duration_from": date_format,
-        "duration_to": date_format,
+        "start_date": date_format,
+        "end_date": date_format,
         "pnp_certificated": yes_no,
         "expiry_date": date_format,
     },
     "info-wp": {
         "work_province": canada_provinces,
         "work_permit_type": wp_apply_wp_type,
-        "duration_from": date_format,
-        "duration_to": date_format,
+        "start_date": date_format,
+        "end_date": date_format,
     },
     "info-visa": {
         "application_purpose": visa_application_purpose,
-        "duration_from": date_format,
-        "duration_to": date_format,
+        "visit_purpose": visit_purpose_5257,
+        "start_date": date_format,
+        "end_date": date_format,
         "funds_available": positive_int_decimal,
     },
     "info-incanadacommon": {
