@@ -1,5 +1,5 @@
-from webform.models.definition import Action, Role
-from webform.models.prmodel import PrModel
+from webform.definition import Action, Role
+from webform.prportal.prmodel import PrModel
 from model.common.address import Addresses
 from model.common.phone import Phones
 from model.common.id import IDs
@@ -8,8 +8,10 @@ from typing import List
 from webform.prportal.data.d0008 import *
 from webform.prportal.data.citizen import citizen
 from webform.prportal.data.country_birth import country_birth
-from webform.prportal.data.country_residence import country_residence
-from webform.models.prmodel import CORs
+from webform.prportal.data.country import country_residence, passport_country
+from webform.prportal.prmodel import CORs
+from utils.utils import best_match
+from datetime import date, timedelta
 
 
 class F0008:
@@ -28,14 +30,14 @@ class F0008:
                 "action_type": "WebPage",
                 "page_name": "Start/Edit for 0008 form",
                 "actions": [],
-                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-imm0008-page1 > div > a.btn.btn-primary",
+                "next_page_tag": "body > pra-root > pra-localized-app > main > div > pra-imm0008-page1 > div > a.btn.btn-primary",
                 "id": "body > pra-root > pra-localized-app > main > div > pra-intake-landing-page > pra-web-form-table > div > table > tbody > tr:nth-child(1) > td:nth-child(5) > button",
             },
             {
                 "action_type": "WebPage",
                 "page_name": "Start/Edit for 0008 form, Continue",
                 "actions": [],
-                "next_page_tag":"#city",
+                "next_page_tag": "#city",
                 "id": "body > pra-root > pra-localized-app > main > div > pra-imm0008-page1 > div > a.btn.btn-primary",
             },
         ]
@@ -61,6 +63,11 @@ class F0008:
 
     # page 2 application details
     def applicationDetails(self):
+        # get best match in standard city list
+        city = best_match(
+            self.pa.prcase.intended_city,
+            city_map[self.pa.prcase.intended_province].keys(),
+        )
         actions = [
             {
                 "action_type": "Select",
@@ -91,9 +98,7 @@ class F0008:
             {
                 "action_type": "Select",
                 "label": "City",
-                "value": city_map[self.pa.prcase.intended_province][
-                    self.pa.prcase.intended_city
-                ],
+                "value": city_map[self.pa.prcase.intended_province][city],
                 "id": "#city",
             },
         ] + self.pdf
@@ -102,13 +107,13 @@ class F0008:
                 "action_type": "WebPage",
                 "page_name": "Application details",
                 "actions": actions,
-                "next_page_tag":"#personalDetailsForm-previouslyMarriedOrCommonLaw-no",
+                "next_page_tag": "#personalDetailsForm-previouslyMarriedOrCommonLaw-no",
                 "id": self.saveContinue(2),
             }
         ]
 
     # page 3 personal details
-    def makePreviousCOR(self,person):
+    def makePreviousCOR(self, person):
         value = []
         cor = person.cor
         cor.pop(0)  # the first line is current cor
@@ -168,6 +173,21 @@ class F0008:
 
     # page 3 personal details
     def personalDetails(self, person: PrModel, role: Role):
+        # current country of residence
+        ccor = CORs(person.cor).current
+        # get best match of selection
+        country_of_birth = best_match(
+            person.personal.country_of_birth, country_birth.keys()
+        )
+
+        citizen1 = best_match(person.personal.citizen, citizen.keys())
+        citizen2 = best_match(person.personal.citizen2, citizen.keys())
+        residence_country = best_match(ccor.country, country_residence.keys())
+        status = best_match(ccor.status, immigration_status.keys())
+        the_marital_status = best_match(
+            person.marriage.marital_status, current_marital_status.keys()
+        )
+
         relationship2pa = {
             "action_type": "Select",
             "label": "Relationship",
@@ -175,7 +195,7 @@ class F0008:
             if person.personal.relationship_to_pa
             else None,
             "id": "#dependantDetailsForm-relationshipToPA",
-            "delay":1500
+            "delay": 1500,
         }
         sp_dp = (
             [
@@ -184,8 +204,7 @@ class F0008:
                     "label": "Accompany to Canada",
                     "id": "#dependantDetailsForm-accompanyingPA-yes"
                     if person.personal.accompany_to_canada
-                    else "#dependantDetailsForm-accompanyingPA-no"
-                    
+                    else "#dependantDetailsForm-accompanyingPA-no",
                 },
                 relationship2pa,
             ]
@@ -219,11 +238,7 @@ class F0008:
         )
 
         dependant_type_set = (
-            [
-                dependant_type_set[0]
-            ]
-            if dependant_type_set
-            else []
+            [dependant_type_set[0]] if dependant_type_set else []
         )  # 如果是spouse，common-law则没有后续选择
         dependant_type_set = (
             dependant_type_set if role != Role.PA else []
@@ -332,7 +347,7 @@ class F0008:
             {
                 "action_type": "Select",
                 "label": "Country of birth",
-                "value": country_birth[person.personal.country_of_birth],
+                "value": country_birth[country_of_birth],
                 "id": "#personalDetailsForm-countryOfBirth",
             },
         ]
@@ -340,30 +355,28 @@ class F0008:
             {
                 "action_type": "Select",
                 "label": "Citizen 1",
-                "value": citizen[person.personal.citizen],
+                "value": citizen[citizen1],
                 "id": "#personalDetailsForm-citizenship1",
             },
             {
                 "action_type": "Select",
                 "label": "Citizen 2",
-                "value": citizen[person.personal.citizen2]
-                if person.personal.citizen2
-                else None,
+                "value": citizen[citizen2] if person.personal.citizen2 else None,
                 "id": "#personalDetailsForm-citizenship2",
             },
         ]
-        ccor = CORs(person.cor).current
+
         current_cor = [
             {
                 "action_type": "Select",
                 "label": "Current country of residence",
-                "value": country_residence[ccor.country],
+                "value": country_residence[residence_country],
                 "id": "#personalDetailsForm-currentCountry",
             },
             {
                 "action_type": "Select",
                 "label": "Status",
-                "value": immigration_status[ccor.status],
+                "value": immigration_status[status],
                 "id": "#personalDetailsForm-immigrationStatus",
             },
         ]
@@ -379,7 +392,9 @@ class F0008:
             {
                 "action_type": "Input",
                 "label": "To",
-                "value": ccor.end_date.strftime("%Y/%m/%d"),
+                "value": ccor.end_date.strftime("%Y/%m/%d")
+                if ccor.end_date
+                else (date.today() + timedelta(days=1)).strftime("%Y/%m/%d"),
                 "id": "#personalDetailsForm-endDateOfImmigrationStatus",
                 "length": 10,
                 "required": True,
@@ -389,8 +404,8 @@ class F0008:
             {
                 "action_type": "Input",
                 "label": "Date of last entry",
-                "value": person.personal.last_entry_date.strftime("%Y/%m/%d")
-                if person.personal.last_entry_date
+                "value": person.status.last_entry_date.strftime("%Y/%m/%d")
+                if person.status.last_entry_date
                 else None,
                 "id": "#personalDetailsForm-dateOfLastEntry",
                 "length": 10,
@@ -399,7 +414,7 @@ class F0008:
             {
                 "action_type": "Input",
                 "label": "Place of last entry",
-                "value": person.personal.last_entry_place,
+                "value": person.status.last_entry_place,
                 "id": "#personalDetailsForm-placeOfLastEntry",
                 "length": 30,
                 "required": True,
@@ -445,14 +460,16 @@ class F0008:
             }
         ]
         previous_cor = (
-            previous_cor + self.makePreviousCOR(person) if has_previous_cor else previous_cor
+            previous_cor + self.makePreviousCOR(person)
+            if has_previous_cor
+            else previous_cor
         )
         # marriage
         marital_status = [
             {
                 "action_type": "Select",
                 "label": "Current marital status",
-                "value": current_marital_status[person.marriage.marital_status],
+                "value": current_marital_status[the_marital_status],
                 "id": "#personalDetailsForm-maritalStatus",
             }
         ]
@@ -501,18 +518,24 @@ class F0008:
                 else "#personalDetailsForm-previouslyMarriedOrCommonLaw-no",
             }
         ]
-        
-        pre_spouse_dob=[{
-                "action_type": Action.Input.value,
-                "label": "Date of birth",
-                "id": "#previousRelationshipForm-previousSpouseDob",
-                "value": person.marriage.pre_sp_dob.strftime("%Y/%m/%d")
-                if person.marriage.pre_sp_dob
-                else None,
-                "length": 10,
-                "required": True,
-            }] if role==Role.PA else []
-        
+
+        pre_spouse_dob = (
+            [
+                {
+                    "action_type": Action.Input.value,
+                    "label": "Date of birth",
+                    "id": "#previousRelationshipForm-previousSpouseDob",
+                    "value": person.marriage.pre_sp_dob.strftime("%Y/%m/%d")
+                    if person.marriage.pre_sp_dob
+                    else None,
+                    "length": 10,
+                    "required": True,
+                }
+            ]
+            if role == Role.PA
+            else []
+        )
+
         pre_marriage = [
             {
                 "action_type": Action.Input.value,
@@ -588,12 +611,16 @@ class F0008:
                 if role == Role.PA
                 else "Dependant personal details",
                 "actions": actions,
-                "next_page_tag":"#contactNo" if role==Role.PA else "#intendedOccupation",
+                "next_page_tag": "#contactNo"
+                if role == Role.PA
+                else "#intendedOccupation",
                 "id": save_continue_id,
             }
         ]
 
     def makeAddress(self, address, type):
+
+        address_country = best_match(address.country, country_residence)
         types = {
             "mailing": {
                 "unit": "#MailingAptUnit",
@@ -624,7 +651,7 @@ class F0008:
                 "id": types[type]["unit"],
                 "value": address.unit if address.unit else None,
                 "length": 10,
-                "required": True,
+                "required": False,
             },
             {
                 "action_type": Action.Input.value,
@@ -653,7 +680,7 @@ class F0008:
             {
                 "action_type": "Select",
                 "label": "Country",
-                "value": country_residence[address.country]
+                "value": country_residence[address_country]
                 if address.country
                 else None,
                 "id": types[type]["country"],
@@ -663,7 +690,7 @@ class F0008:
             {
                 "action_type": "Select",
                 "label": "Province",
-                "value": province_map[address.province] if address.province else None,
+                "value": province_map[address.province] if address.province and address.country.lower()=="canada" else None,
                 "id": types[type]["province"],
             }
         ]
@@ -672,7 +699,7 @@ class F0008:
                 "action_type": Action.Input.value,
                 "label": "Post code",
                 "id": types[type]["post_code"],
-                "value": address.post_code if address.post_code else None,
+                "value": address.post_code.strip() if address.post_code else None,
                 "length": 7,
                 "required": True,
             }
@@ -684,7 +711,7 @@ class F0008:
                 "id": types[type]["district"],
                 "value": address.district if address.district else None,
                 "length": 100,
-                "required": True,
+                "required": False,
             }
         ]
         the_address += (
@@ -792,13 +819,15 @@ class F0008:
                 "action_type": "WebPage",
                 "page_name": "Page 4: Contact information ",
                 "actions": actions,
-                "next_page_tag":'#validPassportNo',
+                "next_page_tag": "#validPassportNo",
                 "id": self.saveContinue(4),
             }
         ]
 
     def passport_id(self, person: PrModel, role: Role):
         ids = IDs(person.personid)
+        the_passport_country = best_match(ids.passport.country, passport_country)
+        the_id_country = best_match(ids.national_id.country, passport_country)
         has_passport = [
             {
                 "action_type": Action.Radio.value,
@@ -809,11 +838,12 @@ class F0008:
             }
         ]
         has_id = [
+            {"action_type": "Wait", "duration": 1500},
             {
                 "action_type": Action.Radio.value,
                 "label": "Has national id",
                 "id": "#NICYes" if ids.national_id.number else "#NICNo",
-            }
+            },
         ]
         passport_content = (
             [
@@ -828,7 +858,7 @@ class F0008:
                 {
                     "action_type": "Select",
                     "label": "Country of issue",
-                    "value": country_residence[ids.passport.country]
+                    "value": passport_country[the_passport_country]
                     if ids.passport.country
                     else None,
                     "id": "#countryOfIssue",
@@ -866,7 +896,7 @@ class F0008:
                 {
                     "action_type": "Select",
                     "label": "Country of issue",
-                    "value": country_residence[ids.national_id.country]
+                    "value": passport_country[the_id_country]
                     if ids.national_id.country
                     else None,
                     "id": "#countryOfIssue",
@@ -911,7 +941,7 @@ class F0008:
                 if role == Role.PA
                 else "Dependant passport",
                 "actions": passport + self.pdf,
-                "next_page_tag":"#NICNo",
+                "next_page_tag": "#NICYes",
                 "id": passport_continue_id,
             },
             {
@@ -920,7 +950,9 @@ class F0008:
                 if role == Role.PA
                 else "Dependant national id",
                 "actions": national_id + self.pdf,
-                "next_page_tag":"#intendedOccupation" if role==Role.PA else "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > pra-imm0008-view-dependant-form > button",
+                "next_page_tag": "#intendedOccupation"
+                if role == Role.PA
+                else "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > pra-imm0008-view-dependant-form > button",
                 "id": national_id_continue_id,
             },
         ]
@@ -976,17 +1008,18 @@ class F0008:
                 if role == Role.PA
                 else "Dependant education",
                 "actions": education_set + self.pdf,
-                "next_page_tag":"#testing-no",
+                "next_page_tag": "#testing-no",
                 "id": save_continue_id,
             }
         ]
 
     def language(self, person: PrModel, role: Role):
+        native_language = best_match(person.personal.native_language, all_languages)
         language = [
             {
                 "action_type": "Select",
                 "label": "Native language",
-                "value": all_languages[person.personal.native_language],
+                "value": all_languages[native_language],
                 "id": "#nativeLanguage",
             },
             {
@@ -1030,7 +1063,9 @@ class F0008:
                 if role == Role.PA
                 else "Dependant language detail",
                 "actions": language + which_one_better + taken_test + self.pdf,
-                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > div.buttons-container > button" if role==Role.PA else "#validPassportNo",
+                "next_page_tag": "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > div.buttons-container > button"
+                if role == Role.PA
+                else "#validPassportNo",
                 "id": save_continue_id,
             }
         ]
@@ -1066,7 +1101,7 @@ class F0008:
                 "action_type": "WebPage",
                 "page_name": "Add dependants or return to applicaton",
                 "actions": actions,
-                "next_page_tag":"#dependantDetailsForm-accompanyingPA-yes",
+                "next_page_tag": "#dependantDetailsForm-accompanyingPA-yes",
                 "id": "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > pra-imm0008-view-dependant-form > button"
                 if self.sp or len(self.dps) > 0
                 else "body > pra-root > pra-localized-app > main > div > pra-imm0008-page9 > div.buttons-container > button",
